<?php

namespace Drupal\patreon_entity\Entity;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\RevisionableContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\user\UserInterface;

/**
 * Defines the Patreon entity entity.
 *
 * @ingroup patreon_entity
 *
 * @ContentEntityType(
 *   id = "patreon_entity",
 *   label = @Translation("Patreon entity"),
 *   bundle_label = @Translation("Patreon entity type"),
 *   handlers = {
 *     "storage" = "Drupal\patreon_entity\PatreonEntityStorage",
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\patreon_entity\PatreonEntityListBuilder",
 *     "views_data" = "Drupal\patreon_entity\Entity\PatreonEntityViewsData",
 *     "translation" = "Drupal\patreon_entity\PatreonEntityTranslationHandler",
 *
 *     "form" = {
 *       "default" = "Drupal\patreon_entity\Form\PatreonEntityForm",
 *       "add" = "Drupal\patreon_entity\Form\PatreonEntityForm",
 *       "edit" = "Drupal\patreon_entity\Form\PatreonEntityForm",
 *       "delete" = "Drupal\patreon_entity\Form\PatreonEntityDeleteForm",
 *     },
 *     "access" = "Drupal\patreon_entity\PatreonEntityAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\patreon_entity\PatreonEntityHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "patreon_entity",
 *   data_table = "patreon_entity_field_data",
 *   revision_table = "patreon_entity_revision",
 *   revision_data_table = "patreon_entity_field_revision",
 *   translatable = TRUE,
 *   admin_permission = "administer patreon entity entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "vid",
 *     "bundle" = "type",
 *     "label" = "name",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "langcode" = "langcode",
 *     "status" = "status",
 *   },
 *   links = {
 *     "canonical" = "/patreon_entity/{patreon_entity}",
 *     "add-page" = "/admin/content/patreon_entity/add",
 *     "add-form" = "/admin/content/patreon_entity/add/{patreon_entity_type}",
 *     "edit-form" = "/patreon_entity/{patreon_entity}/edit",
 *     "delete-form" = "/patreon_entity/{patreon_entity}/delete",
 *     "version-history" = "/patreon_entity/{patreon_entity}/revisions",
 *     "revision" = "/patreon_entity/{patreon_entity}/revisions/{patreon_entity_revision}/view",
 *     "revision_revert" = "/patreon_entity/{patreon_entity}/revisions/{patreon_entity_revision}/revert",
 *     "translation_revert" = "/patreon_entity/{patreon_entity}/revisions/{patreon_entity_revision}/revert/{langcode}",
 *     "revision_delete" = "/patreon_entity/{patreon_entity}/revisions/{patreon_entity_revision}/delete",
 *     "collection" = "/admin/content/patreon_entity",
 *   },
 *   bundle_entity_type = "patreon_entity_type",
 *   field_ui_base_route = "entity.patreon_entity_type.edit_form"
 * )
 */
class PatreonEntity extends RevisionableContentEntityBase implements PatreonEntityInterface {

  use EntityChangedTrait;

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += [
      'user_id' => \Drupal::currentUser()->id(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);

    foreach (array_keys($this->getTranslationLanguages()) as $langcode) {
      $translation = $this->getTranslation($langcode);

      // If no owner has been set explicitly, make the anonymous user the owner.
      if (!$translation->getOwner()) {
        $translation->setOwnerId(0);
      }
    }

    // If no revision author has been set explicitly, make the patreon_entity
    // owner the revision author.
    if (!$this->getRevisionUser()) {
      $this->setRevisionUserId($this->getOwnerId());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getType() {
    return $this->bundle();
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isPublished() {
    return (bool) $this->getEntityKey('status');
  }

  /**
   * {@inheritdoc}
   */
  public function setPublished($published) {
    $this->set('status', $published ? TRUE : FALSE);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of author of the Patreon entity entity.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setDescription(t('The name of the Patreon entity entity.'))
      ->setRevisionable(TRUE)
      ->setSettings([
        'max_length' => 50,
        'text_processing' => 0,
      ])
      ->setDefaultValue('')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -4,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -4,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Publishing status'))
      ->setDescription(t('A boolean indicating whether the Patreon entity is published.'))
      ->setRevisionable(TRUE)
      ->setDefaultValue(TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));

    $fields['revision_translation_affected'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Revision translation affected'))
      ->setDescription(t('Indicates if the last edit of a translation belongs to current revision.'))
      ->setReadOnly(TRUE)
      ->setRevisionable(TRUE)
      ->setTranslatable(TRUE);

    return $fields;
  }

}
