<?php

namespace Drupal\patreon;

use \Patreon\API;
use \Patreon\OAuth;
use Drupal\Core\Url;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use \Drupal\user\UserInterface;

/**
 * Class PatreonService.
 *
 * @package Drupal\patreon
 */
class PatreonService implements PatreonServiceInterface {

  use StringTranslationTrait;

  /**
   * Constructs a new PatreonService object.
   */
  public function __construct() {

  }

  /**
   * The API token for this Patreon Service API connection.
   *
   * @var string
   *    The API token for this Patreon Service API connection.
   */
  private $token = '';

  /**
   * Bool to capture whether a token refresh has been tried.
   *
   * @var bool
   */
  private $refreshTried = FALSE;

  /**
   * {@inheritdoc}
   */
  public function getToken() {
    if (!$this->token) {
      throw new PatreonMissingTokenException('An API token has not been set.');
    }
    return $this->token;
  }

  /**
   * {@inheritdoc}
   */
  public function setToken($token) {
    $this->token = $token;
  }

  /**
   * {@inheritdoc}
   */
  public function getCallback() {
    return Url::fromRoute('patreon.patreon_controller_oauth_callback', array(), array('absolute' => TRUE));
  }

  /**
   * {@inheritdoc}
   */
  public function authoriseAccount($client_id, $redirect = TRUE) {
    $redirect_url = $this->getCallback();
    $url = Url::fromUri(PATREON_URL . '/oauth2/authorize?response_type=code&client_id=' . $client_id . '&redirect_uri=' . $redirect_url->toString());

    if ($redirect) {
      return new TrustedRedirectResponse($url->toString());
    }
    else {
      return $url;
    }
  }

  /**
   * {@inheritdoc}
   */
  private function getOauth() {
    $config = \Drupal::config('patreon.settings');
    $key = $config->get('patreon_client_id');
    $secret = $config->get('patreon_client_secret');
    return new \Patreon\OAuth($key, $secret);
  }

  /**
   * {@inheritdoc}
   */
  public function getRefreshedTokens($token, $redirect) {
    $oauth = $this->getOauth();
    $tokens = $oauth->refresh_token($token, $redirect);

    if (array_key_exists('error', $tokens)) {
      if ($tokens['error'] == 'access_denied') {
        throw new PatreonUnauthorizedException($tokens['error']);
      }
      else {
        throw new PatreonGeneralException($tokens['error']);
      }
    }

    return $tokens;
  }

  /**
   * {@inheritdoc}
   */
  public function tokensFromCode($code) {
    $oauth = $this->getOauth();
    $url = $this->getCallback();
    $tokens = $oauth->get_tokens($code, $url->toString());

    if (array_key_exists('error', $tokens)) {
      if ($tokens['error'] == 'access_denied') {
        throw new PatreonUnauthorizedException($tokens['error']);
      }
      else {
        throw new PatreonGeneralException($tokens['error']);
      }
    }

    return $tokens;
  }

  /**
   * {@inheritdoc}
   */
  public function storeTokens($tokens, UserInterface $account = NULL) {
    $updated_config = \Drupal::service('config.factory')
      ->getEditable('patreon.settings');
    $updated_config->set('patreon_access_token', $tokens['access_token']);
    $updated_config->set('patreon_refresh_token', $tokens['refresh_token'])->save();
  }

  /**
   * {@inheritdoc}
   */
  public function getStoredTokens(UserInterface $account = NULL) {
    $return = array();
    $config = \Drupal::config('patreon.settings');
    $return['refresh_token'] = $config->get('patreon_refresh_token');
    $return['access_token'] = $config->get('patreon_access_token');

    return $return;
  }

  /**
   * {@inheritdoc}
   */
  public function checkResponse($response) {
    if (isset($response['errors']) && !empty($response['errors'])) {
      if ($response['errors'][0]['code_name'] == 'Unauthorized') {
        throw new PatreonUnauthorizedException('Patreon API has returned that the user is unauthorized for that action.');
      }
      else {
        throw new PatreonGeneralException($response['errors'][0]['detail']);
      }
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function fetchUser() {
    return $this->apiFetch('fetch_user');
  }

  /**
   * {@inheritdoc}
   */
  public function fetchCampaign() {
    return $this->apiFetch('fetch_campaign');
  }

  /**
   * {@inheritdoc}
   */
  public function fetchCampaignAndPatrons() {
    return $this->apiFetch('fetch_campaign_and_patrons');
  }

  /**
   * {@inheritdoc}
   */
  public function fetchPagePledges($campaign_id, $page_size, $cursor = NULL) {
    return $this->apiFetch('fetch_page_of_pledges', array(
      $campaign_id,
      $page_size,
      $cursor,
    ));
  }

  /**
   * Helper function to query the Patreon API.
   *
   * @param string $function
   *   A valid Patreon API function.
   * @param array $parameters
   *   An array of parameters required for the function call. Defaults to empty.
   *
   * @return null|array
   *   An array of the function callback data, or NULL on error.
   */
  private function apiFetch($function, $parameters = array()) {
    $return = NULL;

    try {
      $client = new \Patreon\API($this->getToken());

      if (method_exists($client, $function)) {
        if ($parameters) {
          list($campaign_id, $page_size, $cursor) = $parameters;
          $response = $client->{$function}($campaign_id, $page_size, $cursor);
        }
        else {
          $response = $client->{$function}();
        }

        if ($this->checkResponse($response)) {
          $return = $response;
        }
      }
    }
    catch (PatreonMissingTokenException $e) {
      $message = $this->t('The Patreon API returned the following error: :error', array(
        ':error' => $e->getMessage(),
      ));
      \Drupal::logger('patreon')->error($message);
      drupal_set_message($this->t('A valid API token has not been set. Please visit @link', array(
        '@link' => Url::fromRoute('patreon.settings_form'),
      )), 'error');
    }
    catch (PatreonUnauthorizedException $e) {
      if (!$this->refreshTried) {
        $tokens = $this->getStoredTokens();
        $token = $tokens['refresh_token'];
        $redirect = $this->getCallback();
        try {
          $this->refreshTried = TRUE;
          $new_tokens = $this->getRefreshedTokens($token, $redirect->toString());
          $this->storeTokens($new_tokens);

          // Retry the function callback.
          $this->setToken($new_tokens['access_token']);
          $return = $this->apiFetch($function, $parameters);
        }
        catch (PatreonUnauthorizedException $error) {
          $message = $this->t('The Patreon API returned the following error: :error', array(
            ':error' => $error->getMessage(),
          ));
          \Drupal::logger('patreon')->error($message);
          drupal_set_message($this->t('Your API token has expired or not been set. Please visit !link', array(
            '!link' => Url::fromRoute('patreon.settings_form'),
          )), 'error');
        }
        catch (PatreonGeneralException $error) {
          $message = $this->t('The Patreon API returned the following error: :error', array(
            ':error' => $error->getMessage(),
          ));
          \Drupal::logger('patreon')->error($message);
          drupal_set_message($message, 'error');
        }
      }
      else {
        $message = $this->t('The Patreon API returned the following error: :error', array(
          ':error' => $e->getMessage(),
        ));
        \Drupal::logger('patreon')->error($message);
        drupal_set_message($this->t('Your API token has expired or not been set. Please visit !link', array(
          '!link' => Url::fromRoute('patreon.settings_form'),
        )), 'error');
      }

    }
    catch (PatreonGeneralException $e) {
      $message = $this->t('The Patreon API returned the following error: :error', array(
        ':error' => $e->getMessage(),
      ));
      \Drupal::logger('patreon')->error($message);
      drupal_set_message($message, 'error');
    }

    return $return;
  }

  /**
   * Helper to check if a Patreon user is a patron of the client.
   *
   * @param array $patreon_account
   *   Results array from patreon_fetch_user().
   *
   * @return bool
   *   TRUE is user's pledges match creator id. Defaults to FALSE.
   */
  public function isPatron($patreon_account) {
    $return = FALSE;
    $config = \Drupal::config('patreon.settings');

    if ($creator_id = $config->get('patreon_creator_id')) {
      if (isset($patreon_account['relationships']['pledges']['data'])) {
        $key = 0;
        while (array_key_exists($key, $patreon_account['relationships']['pledges']['data']) && $return == FALSE) {
          $pledge = $patreon_account['relationships']['pledges']['data'][$key];
          if (isset($pledge['relationships']['creator']['data']['id'])) {
            if ($pledge['relationships']['creator']['data']['id'] == $creator_id) {
              $return = TRUE;
            }
          }

          $key++;
        }
      }
    }

    return $return;
  }

  /**
   * {@inheritdoc}
   */
  public function getValueByKey(array $haystack, $needle) {
    $found = FALSE;
    $iterator = new \RecursiveArrayIterator($haystack);
    $return = $this->iteratorSearch($iterator, $needle, $found);
    return $return;
  }

  /**
   * Helper to recurse through a RecursiveArrayIterator for a value.
   *
   * @param \RecursiveArrayIterator $iterator
   *   A valid RecursiveArrayIterator.
   * @param string $needle
   *   A key to search the array for.
   * @param bool $found
   *   Whether value has been found.
   *
   * @return mixed
   *   The value of the key in the array. NULL if not found.
   */
  private function iteratorSearch(\RecursiveArrayIterator $iterator, $needle, &$found) {
    $return = NULL;
    while ($iterator->valid() && !$found) {
      if ($iterator->key() == $needle) {
        $return = $iterator->current();
        $found = TRUE;
      }
      elseif ($iterator->hasChildren()) {
        $return = $this->iteratorSearch($iterator->getChildren(), $needle, $found);
      }

      $iterator->next();
    }

    return $return;
  }

}
